/**
 * CMI : Cluster Method Invocation
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: carol@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * --------------------------------------------------------------------------
 * $Id: ImmutableClusteredObjectInfos.java 1965 2008-07-29 15:40:02Z loris $
 * --------------------------------------------------------------------------
 */

package org.ow2.cmi.controller.common;

import java.io.Serializable;
import java.util.Set;

import net.jcip.annotations.Immutable;

/**
 * @author Loris Bouzonnet
 */
@Immutable
public class ImmutableClusteredObjectInfos implements Cloneable, Serializable{

    /**
     *
     */
    private static final long serialVersionUID = 3358806293593264837L;

    /**
     * Name of the interface that the object implements.
     */
    private final String clusterName;

    /**
     * True if a CMI proxy should be used to make invocations.
     */
    private final boolean proxyEnabled;

    /**
     * Name of the distributed object.
     */
    private final String objectName;

    /**
     * Name of the interface that the object implements.
     */
    private final String itfName;

    /**
     * Name of the business interface that the object implements.
     */
    private final String businessName;

    /**
     * True if this object has a state.
     */
    private final boolean stateful;

    /**
     * Classnames of the application exceptions.
     */
    private final Set<String> applicationExceptionNames;

    public ImmutableClusteredObjectInfos(
            final String clusterName,
            final boolean proxyEnabled,
            final String objectName,
            final String itfName,
            final String businessName,
            final boolean stateful,
            final Set<String> applicationExceptionNames) {
        this.clusterName = clusterName;
        this.proxyEnabled = proxyEnabled;
        this.objectName = objectName;
        this.itfName = itfName;
        this.businessName = businessName;
        this.stateful = stateful;
        this.applicationExceptionNames = applicationExceptionNames;
    }

    /**
     * @return a name of the interface that the object implements
     */
    public String getClusterName() {
        return clusterName;
    }

    /**
     * @return true if a CMI proxy should be used to make invocations
     */
    public boolean isProxyEnabled() {
        return proxyEnabled;
    }

    /**
     * @return Name of the interface that the object implements.
     */
    public String getItfName() {
        return itfName;
    }

    /**
     * @return the businessName
     */
    public String getBusinessName() {
        return businessName;
    }

    /**
     * @return the objectName
     */
    public String getObjectName() {
        return objectName;
    }

    /**
     * Return true if this object has a state.
     * @return true if this object has a state
     */
    public boolean isStateful() {
        return stateful;
    }

    /**
     * @return classnames of the application exceptions
     */
    public Set<String> getApplicationExceptionNames() {
        return applicationExceptionNames;
    }

    @Override
    public boolean equals(final Object object) {
        if(object == null && !(object instanceof ImmutableClusteredObjectInfos)) {
            return false;
        }
        ImmutableClusteredObjectInfos immutableInfos = (ImmutableClusteredObjectInfos) object;
        return (clusterName == null ?
                    immutableInfos.clusterName == null :
                        clusterName.equals(immutableInfos.clusterName))
            && (objectName == null ?
                    immutableInfos.objectName == null :
                        objectName.equals(immutableInfos.objectName))
            && (itfName == null ?
                    immutableInfos.itfName == null :
                        itfName.equals(immutableInfos.itfName))
            && (businessName == null ?
                    immutableInfos.businessName == null :
                        businessName.equals(immutableInfos.businessName))
            && stateful == immutableInfos.stateful
            && (applicationExceptionNames == null ?
                    immutableInfos.applicationExceptionNames == null :
                        applicationExceptionNames.equals(immutableInfos.applicationExceptionNames))
            && proxyEnabled == immutableInfos.proxyEnabled;
    }

    @Override
    public int hashCode() {
        return (itfName == null ? 0 : itfName.hashCode())
        + (businessName == null ? 0 : businessName.hashCode())
        + (clusterName == null ?0 : clusterName.hashCode())
        + (objectName == null ? 0 : objectName.hashCode())
        + Boolean.valueOf(stateful).hashCode()
        + (applicationExceptionNames == null ? 0 : applicationExceptionNames.hashCode())
        + Boolean.valueOf(proxyEnabled).hashCode();
    }

    @Override
    public String toString() {
        return "ImmutableCusteredObjectInfos[clusterName=" + clusterName
            + ",objectName=" + objectName
            + ",itfName=" + itfName
            + ",businessName=" + businessName
            + ",stateful=" + stateful
            + ",applicationExceptionNames=" + applicationExceptionNames
            + ",proxyEnabled=" + proxyEnabled
            + "]";
    }

}
