/**
 * CMI : Cluster Method Invocation
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: carol@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * --------------------------------------------------------------------------
 * $Id:IPolicyStrategyManager.java 1836 2008-06-10 15:39:15Z eyindanga $
 * --------------------------------------------------------------------------
 */

package org.ow2.cmi.controller.server;

import java.util.Collection;

import org.ow2.cmi.lb.policy.IPolicy;
import org.ow2.cmi.lb.strategy.IStrategy;
import org.ow2.cmi.reference.ObjectNotFoundException;


/**
 * Interface for strategy/policy managers. The server-side manager delegates
 * management of policies and strategies to this manager.
 *
 * @author The new CMI team
 *
 */
public interface IPolicyStrategyManager {
    /**
     * Returns the class that defines the policy for an object with the given
     * name.
     *
     * @param objectName
     *            a name of object
     * @return the class that defines the policy for an object with the given
     *         name
     * @throws ObjectNotFoundException
     *             if none object has the given name
     * @throws ServerClusterViewManagerException
     *             if the class cannot be loaded
     */
    Class<? extends IPolicy<?>> getPolicyClass(final String objectName)
            throws ObjectNotFoundException, ServerClusterViewManagerException;

    /**
     * Returns the class that defines the strategy for an object with the given
     * name.
     *
     * @param objectName
     *            a name of object
     * @return the class that defines the strategy for an object with the given
     *         name
     * @throws ObjectNotFoundException
     *             if none object has the given name
     * @throws ServerClusterViewManagerException
     *             if the class cannot be loaded
     */
    Class<? extends IStrategy<?>> getStrategyClass(final String objectName)
            throws ObjectNotFoundException, ServerClusterViewManagerException;

    /**
     * Receives archive containing load balancing classes. The server cluster
     * view manager delegate the call to the policy/Strategy manager. Since
     * complex treatment might be performed, to assume the reception of the
     * archive.
     *
     * @param bytesOfFile
     *            Bytes content of the file
     * @param location
     *            File location
     * @param strategies policies exported by the archive
     * @param policies strategies exported by the archive
     * @return The archive Id used by <code>PolicyStrategyManager</code> to
     *         register the new archive.
     */
    Object addLoadBalancingArchive(final byte[] bytesOfFile,
            final String location, Class<?>[] policies, Class<?>[] strategies);

    /**
     * Add a strategy class
     *
     * @param strategyClass
     *            the strategy class to add.
     * @param overwrite
     *            if true then existing class with the given name be
     *            overwritten.
     * @throws ServerClusterViewManagerException,
     *             if <code>strategyClass</code> cannot be added.
     */
    void addStrategyClass(final Class<? extends IStrategy<?>> strategyClass,
            final boolean overwrite) throws ServerClusterViewManagerException;

    /**
     * Add a policy class
     *
     * @param policyClass
     *            the policy class to add.
     * @param overwrite
     *            if true then existing class with the given name be
     *            overwritten.
     * @throws ServerClusterViewManagerException,
     *             if <code>policyClass</code> cannot be added.
     */
    void addPolicyClass(final Class<? extends IPolicy<?>> policyClass,
            final boolean overwrite) throws ServerClusterViewManagerException;

    /**
     * Remove Load Balancing Archive, previously added.
     * @param archiveId The Id of the archive.
     * @param policies names of the policies to remove
     * @param strategies names of the policies to remove
     * @param isArchiveOwner if true then the call is performed by the archive owner.
     * The archive owner is the first server that deployed the archive.
     * @throws Exception
     *
     */
    public void removeLoadBalancingArchive(final Object archiveId, final String[] policies,
            final String[] strategies, final boolean isArchiveOwner) throws Exception;

    /**
     *Add default policy for the <code>objectName</code> parameter.
     * @param objectName name of the object.
     * @param policyClassName the policy name
     */
    public void addDefaultPolicyClassName(final String objectName, final String policyClassName);

    /**
     *Add default strategy for the <code>objectName</code> parameter.
     * @param objectName name of the object.
     * @param strategyClassName the policy name
     */
    public void addDefaultStrategyClassName(final String objectName, final String strategyClassName);

    /**
     * Get all the policies
     * @return the policy class names.
     */
    Collection<? extends String> getPolicyClassNames();

    /**
     * Get all the strategies
     * @return the strategy class names.
     */
    Collection<? extends String> getStrategyClassNames();

    /**
     *	Load classes embedded in the CMI distribution.
     */
    void loadEmbeddedLBClasses();

    /**
     * Return true if the given class name is embedded policy class.
     * @param className a name of class
     * @return true if the given class name is embedded policy class
     */
    boolean isEmbeddedPolicy(final String className);


    /**
     * Return true if the given class name is embedded strategy class.
     * @param className a name of class
     * @return true if the given class name is embedded strategy class
     */
    boolean isEmbeddedStrategy(final String className);

}
